@@ -567,7 +567,7 @@ namespace code {
 			assert(instr->dest().type() == opRegister);
 			assert(same(ptrA, instr->dest().reg()));
 			if (instr->size() == Size::sByte) {
-				to->putByte(0x98); // CBW
+				to->putByte(0x66); to->putByte(0x98); // (size override) CBW
 				to->putByte(0xF6); // DIV
 			} else {
 				to->putByte(0x99); // CDQ
