@@ -1,6 +1,7 @@
 #include "stdafx.h"
 #include "Serialization.h"
 #include "Str.h"
+#include "StrBuf.h"
 #include "Exception.h"
 
 namespace storm {
@@ -64,17 +65,33 @@ namespace storm {
 	}
 
 
-	SerializedMember::SerializedMember(Str *name, Type *type) : name(name), type(type) {}
+	SerializedMember::SerializedMember(Str *name, Type *type)
+		: name(name), type(type), init(null) {}
+
+	SerializedMember::SerializedMember(Str *name, Type *type, MAYBE(FnBase *) init)
+		: name(name), type(type), init(init) {}
 
 	SerializedStdType::SerializedStdType(Type *t, FnBase *ctor)
-		: SerializedType(t, ctor), names(new (engine()) Array<Str *>()) {}
+		: SerializedType(t, ctor),
+		  names(new (engine()) Array<Str *>()),
+		  inits(new (engine()) Array<FnBase *>()) {}
 
 	SerializedStdType::SerializedStdType(Type *t, FnBase *ctor, Type *parent)
-		: SerializedType(t, ctor, parent), names(new (engine()) Array<Str *>()) {}
+		: SerializedType(t, ctor, parent),
+		  names(new (engine()) Array<Str *>()),
+		  inits(new (engine()) Array<FnBase *>()) {}
 
 	void SerializedStdType::add(Str *name, Type *type) {
 		typeAdd(type);
 		names->push(name);
+		inits->push(null);
+		typeRepeat(typeCount());
+	}
+
+	void SerializedStdType::add(Str *name, Type *type, MAYBE(FnBase *) init) {
+		typeAdd(type);
+		names->push(name);
+		inits->push(init);
 		typeRepeat(typeCount());
 	}
 
@@ -83,7 +100,7 @@ namespace storm {
 	}
 
 	SerializedMember SerializedStdType::at(Nat i) const {
-		return SerializedMember(names->at(i), typeAt(i));
+		return SerializedMember(names->at(i), typeAt(i), inits->at(i));
 	}
 
 	typeInfo::TypeInfo SerializedStdType::baseInfo() const {
@@ -138,9 +155,11 @@ namespace storm {
 	 * ObjIStream
 	 */
 
-	ObjIStream::Member::Member(Str *name, Nat type) : name(name), type(type), read(0) {}
+	ObjIStream::Member::Member(Str *name, Nat type) : type(type), read(0), data(name) {}
 
-	ObjIStream::Member::Member(const Member &o, Int read) : name(o.name), type(o.type), read(read) {}
+	ObjIStream::Member::Member(FnBase *init) : type(0), read(-1), data(init) {}
+
+	ObjIStream::Member::Member(const Member &o, Int read) : data(o.data), type(o.type), read(read) {}
 
 
 	ObjIStream::Desc::Desc(Byte flags, Nat parent, Str *name) : data(Nat(flags) << 24), parent(parent) {
@@ -148,10 +167,10 @@ namespace storm {
 
 		Type *t = runtime::fromIdentifier(name);
 		if (!t)
-			throw new (this) SerializationError(TO_S(this, S("Unknown type: ") << demangleName(name)));
+			throw new (this) SerializationFormatError(TO_S(this, S("Unknown type: ") << demangleName(name)));
 		const Handle &h = runtime::typeHandle(t);
 		if (!h.serializedTypeFn)
-			throw new (this) SerializationError(TO_S(this, S("The type ") << demangleName(name) << S(" is not serializable.")));
+			throw new (this) SerializationFormatError(TO_S(this, S("The type ") << demangleName(name) << S(" is not serializable.")));
 
 		info = (*h.serializedTypeFn)();
 	}
@@ -162,9 +181,11 @@ namespace storm {
 	}
 
 	Nat ObjIStream::Desc::findMember(Str *name) const {
-		for (Nat i = 0; i < members->count(); i++)
-			if (*members->at(i).name == *name)
+		for (Nat i = 0; i < members->count(); i++) {
+			Str *mName = members->at(i).name();
+			if (mName && *mName == *name)
 				return i;
+		}
 		return members->count();
 	}
 
@@ -200,6 +221,69 @@ namespace storm {
 	}
 
 
+	/**
+	 * Size limits.
+	 */
+
+	// Check if 'base + add' would be larger than 'limit'. Updates 'base' to reflect the addition,
+	// but caps it at 'limit' to avoid overflows.
+	static bool addSize(Nat &base, Nat add, Nat limit) {
+		// Note: It is safe to rely on overflow here since unsigned types are defined to behave as
+		// modular arithmetics.
+		Nat original = base;
+		base += add;
+		if (base > limit || base < original) {
+			base = limit;
+			return true;
+		}
+		return false;
+	}
+
+	static void throwLimitError(Engine &e, Nat limit) {
+		throw new (e) SizeLimitReached(S("type metadata"), 0, limit);
+	}
+
+	void ObjIStream::maxTypeDescSize(Nat limit) {
+		typeDescLimit = limit;
+		if (typeDescCurrent >= limit)
+			throwLimitError(engine(), limit);
+	}
+
+	void ObjIStream::checkTypeDescSize(Nat add) {
+		if (addSize(typeDescCurrent, add, typeDescLimit))
+			throwLimitError(engine(), typeDescLimit);
+	}
+
+	static void throwSizeError(Engine &e, Nat alloc, Nat limit) {
+		throw new (e) SizeLimitReached(S("an object"), alloc, limit);
+	}
+
+	void ObjIStream::checkAllocSize(Nat add) {
+		if (add > readSizeBudget)
+			throwSizeError(engine(), add, readSizeBudget);
+		readSizeBudget -= add;
+	}
+
+	static void throwArraySizeError(Engine &e, Nat alloc, Nat limit) {
+		throw new (e) SizeLimitReached(S("an array"), alloc, limit);
+	}
+
+	void ObjIStream::checkArrayAlloc(Nat elemSize, Nat count) {
+		Word totalSizeW = Word(elemSize) * Word(count);
+		if (totalSizeW > maxArraySize)
+			throwArraySizeError(engine(), Nat(min(Word(0xFFFFFFFF), totalSizeW)), maxArraySize);
+
+		Nat totalSize = Nat(totalSizeW);
+		if (totalSize > readSizeBudget)
+			throwArraySizeError(engine(), Nat(totalSize), readSizeBudget);
+
+		readSizeBudget -= totalSize;
+	}
+
+	/**
+	 * IStream.
+	 */
+
 	template <class T, T (IStream::* readFn)()>
 	static void CODECALL read(T *out, ObjIStream *from) {
 		new (out) T((from->from->*readFn)());
@@ -207,11 +291,17 @@ namespace storm {
 	}
 
 	static void readStr(Str **out, ObjIStream *from) {
-		new (Place(out)) Str(from->from);
-		from->end();
+		new (Place(out)) Str(from);
 	}
 
-	ObjIStream::ObjIStream(IStream *src) : from(src) {
+	ObjIStream::ObjIStream(IStream *src)
+		: from(src),
+		  maxReadSize(0xFFFFFFFF),
+		  maxArraySize(0xFFFFFFFF),
+		  typeDescLimit(0xFFFFFFFF),
+		  typeDescCurrent(0),
+		  readSizeBudget(0) {
+
 		clearObjects();
 
 		depth = new (this) Array<Cursor>();
@@ -242,7 +332,13 @@ namespace storm {
 	}
 
 	void ObjIStream::readValue(Type *type, PTR_GC out) {
-		Info info = start();
+		Info info = start(out);
+
+		// Default initialized?
+		if (info.expectedType == endId)
+			return;
+
+		// Read from storage?
 		if (info.result.any()) {
 			// TODO: Check type!
 			info.result.moveValue(out);
@@ -251,11 +347,11 @@ namespace storm {
 
 		Desc *expected = findInfo(info.expectedType);
 		if (!expected->isValue())
-			throw new (this) SerializationError(S("Expected a value type, but got a class type."));
+			throw new (this) SerializationFormatError(S("Expected a value type, but got a class type."));
 		if (expected->info->type != type) {
 			Str *msg = TO_S(this, S("Type mismatch. Expected ") << runtime::typeName(expected->info->type)
 							<< S(" but got ") << runtime::typeName(type) << S("."));
-			throw new (this) SerializationError(msg);
+			throw new (this) SerializationFormatError(msg);
 		}
 
 		readValueI(expected, out);
@@ -266,10 +362,21 @@ namespace storm {
 		// since the types used at the root of deserialization do not need to be equal. However,
 		// 'type' needs to be a parent of whatever we find as 'actual' later on, which is a subclass
 		// of 'expected'.
-		Info info = start();
+		Info info = start(null);
+
+		// Read from cache? Note: we don't have to care about special cases here like in 'readValue'.
 		if (info.result.any()) {
-			// TODO: Check type!
-			return (Object *)info.result.getObject();
+			RootObject *result = info.result.getObject();
+			if (!runtime::isA(result, type)) {
+				StrBuf *msg = new (this) StrBuf();
+				*msg << S("Attempted to read an object of type: ")
+					 << runtime::typeName(type)
+					 << S(" but received an object of type ")
+					 << runtime::typeName(runtime::typeOf(result))
+					 << S(".");
+				throw new (this) SerializationFormatError(msg->toS());
+			}
+			return (Object *)result;
 		}
 
 		Desc *expected = findInfo(info.expectedType);
@@ -286,12 +393,23 @@ namespace storm {
 	}
 
 	void ObjIStream::readPrimitiveValue(StoredId id, PTR_GC out) {
-		Info info = start();
+		Info info = start(out);
+
+		// Using default initialization? If so, we are already done.
+		if (info.expectedType == endId)
+			return;
+
+		// Stored value?
+		if (info.result.any()) {
+			info.result.moveValue(out);
+			return;
+		}
+
 		Desc *expected = findInfo(info.expectedType);
 		if (!expected->isValue())
-			throw new (this) SerializationError(S("Expected a value type, but got a class type."));
+			throw new (this) SerializationFormatError(S("Expected a value type, but got a class type."));
 		if (id != info.expectedType)
-			throw new (this) SerializationError(S("Mismatch of built-in types!"));
+			throw new (this) SerializationFormatError(S("Mismatch of built-in types!"));
 
 		if (info.result.any()) {
 			// TODO: Check type!
@@ -303,24 +421,26 @@ namespace storm {
 	}
 
 	Object *ObjIStream::readPrimitiveObject(StoredId id) {
-		Info info = start();
-		Desc *expected = findInfo(info.expectedType);
-		if (expected->isValue())
-			throw new (this) SerializationError(S("Expected a class type, but got a value type."));
-		if (id != info.expectedType)
-			throw new (this) SerializationError(S("Mismatch of built-in types!"));
+		Info info = start(null);
 
+		// Default initialization, or reading duplicates?
 		if (info.result.any()) {
 			// TODO: Check type?
 			return (Object *)info.result.getObject();
 		}
 
+		Desc *expected = findInfo(info.expectedType);
+		if (expected->isValue())
+			throw new (this) SerializationFormatError(S("Expected a class type, but got a value type."));
+		if (id != info.expectedType)
+			throw new (this) SerializationFormatError(S("Mismatch of built-in types!"));
+
 		Object *created = (Object *)runtime::allocObject(0, expected->info->type);
 		readValueI(expected, created);
 		return created;
 	}
 
-	Variant ObjIStream::readClass(Nat typeId) {
+	Variant ObjIStream::readObject(Nat typeId) {
 		Desc *type = findInfo(typeId);
 		if (type->isValue()) {
 			Type *t = type->info->type;
@@ -336,7 +456,16 @@ namespace storm {
 				return Variant(created);
 			}
 		} else {
-			return Variant(readClassI(type, type->info->type));
+			// Note: Primitives are never duplicated, so we need to check if it is a user-type or a
+			// primitive type. This likely means that we don't de-duplicate strings. It is fine
+			// since strings are immutable, but it could be a bit wasteful.
+			if (typeId < firstCustomId) {
+				Object *created = (Object *)runtime::allocObject(0, type->info->type);
+				readValueI(type, created);
+				return Variant(created);
+			} else {
+				return Variant(readClassI(type, type->info->type));
+			}
 		}
 	}
 
@@ -363,7 +492,7 @@ namespace storm {
 		Nat objId = from->readNat();
 		if (Object *old = objIds->get(objId, null)) {
 			if (!runtime::isA(old, t))
-				throw new (this) SerializationError(S("Wrong type found during deserialization."));
+				throw new (this) SerializationFormatError(S("Wrong type found during deserialization."));
 			return old;
 		}
 
@@ -373,10 +502,14 @@ namespace storm {
 
 		// Make sure the type we're about to create is appropriate.
 		if (!runtime::isA(actual->info->type, t))
-			throw new (this) SerializationError(S("Wrong type found during deserialization."));
+			throw new (this) SerializationFormatError(S("Wrong type found during deserialization."));
+
+		// Check so that we are not over capacity.
+		Type *type = actual->info->type;
+		checkAllocSize(runtime::typeGc(type)->stride);
 
 		// Allocate the object and deserialize by calling the constructor, just like we do with value types.
-		Object *created = (Object *)runtime::allocObject(0, actual->info->type);
+		Object *created = (Object *)runtime::allocObject(0, type);
 		objIds->put(objId, created);
 
 		readValueI(actual, created);
@@ -384,12 +517,16 @@ namespace storm {
 		return created;
 	}
 
-	ObjIStream::Info ObjIStream::start() {
+	ObjIStream::Info ObjIStream::start(PTR_GC valueOut) {
 		Info r = { 0, Variant() };
 
 		if (depth->empty()) {
-			// First type. Read its id from the stream.
+			// First type.
 
+			// Reset current size limit.
+			readSizeBudget = maxReadSize;
+
+			// Read its id from the stream.
 			if (!from->more())
 				throw new (this) EndOfStream();
 
@@ -406,38 +543,66 @@ namespace storm {
 		// Some other type. Examine what we expect to read.
 		Cursor &at = depth->last();
 		if (at.customDesc())
-			throw new (this) SerializationError(S("Can not use 'start' when serializing custom types."));
+			throw new (this) SerializationFormatError(S("Can not use 'start' when serializing custom types."));
 
 		// Process objects until we find something we can return!
 		while (true) {
 			if (!at.any())
-				throw new (this) SerializationError(S("Trying to deserialize too many members."));
+				throw new (this) SerializationFormatError(S("Trying to deserialize too many members."));
 
 			const Member &expected = at.current();
 			at.next();
 
-			if (expected.read < 0) {
-				// Read one object into temporary storage and continue.
-				at.pushTemporary(readClass(expected.type));
-			} else if (expected.read > 0) {
-				r.expectedType = expected.type;
-				r.result = at.temporary(Nat(expected.read - 1));
-				return r;
-			} else {
+			if (expected.read == 0) {
 				// Read it now!
 				r.expectedType = expected.type;
 				return r;
+			} else if (FnBase *init = expected.init()) { // 'read' is -1
+				// Indicate that we should use the default value. Return expected type = 0 to
+				// indicate this, put the result either in 'valueOut' or in the variant.
+				r.expectedType = endId;
+
+				if (valueOut) {
+					void *params[1] = {};
+					init->rawCall().call(init, valueOut, params);
+				} else {
+					os::FnCall<Object *, 1> params = os::fnCall();
+					r.result = Variant(init->callRaw(params, null, null));
+				}
+
+				return r;
+			} else if (expected.read == -2) {
+				// Read one object into temporary storage and continue.
+				at.pushTemporary(readObject(expected.type));
+			} else if (expected.read < -2) {
+				// Read one object, and ignore it.
+				readObject(expected.type);
+			} else /* if (expected.read > 0) */ {
+				// Retrieve a value from temporary storage.
+				r.expectedType = expected.type;
+				r.result = at.temporary(Nat(expected.read - 1));
+				return r;
 			}
 		}
 	}
 
 	void ObjIStream::end() {
 		if (depth->empty())
-			throw new (this) SerializationError(S("Mismatched calls to startX during dedeserialization!"));
+			throw new (this) SerializationFormatError(S("Mismatched calls to startX during dedeserialization!"));
 
-		Cursor end = depth->last();
-		if (!end.atEnd())
-			throw new (this) SerializationError(S("Missing fields during deserialization!"));
+		Cursor &at = depth->last();
+
+		// It is possible that there are things we should ignore at the end of the current data.
+		while (!at.atEnd()) {
+			const Member &member = at.current();
+			at.next();
+
+			if (member.read >= -2)
+				throw new (this) SerializationFormatError(S("Missing fields in the read constructor during serialization!"));
+
+			// Now, member.read is equal to -3, so ignore whatever we find.
+			readObject(member.type);
+		}
 
 		depth->pop();
 
@@ -453,24 +618,28 @@ namespace storm {
 			return result;
 
 		Byte flags = from->readByte();
-		Str *name = Str::read(from);
+		Str *name = Str::read(from, typeDescLimit - typeDescCurrent);
 		Nat parent = from->readNat();
 
+		checkTypeDescSize(sizeof(Desc) + name->peekLength()*sizeof(wchar));
 		result = new (this) Desc(flags, parent, name);
 
 		if (flags & typeInfo::tuple) {
 			// Tuple.
-			result->members->push(Member(null, natId));
+			checkTypeDescSize(sizeof(Member));
+			result->members->push(Member((Str *)null, natId));
 
 			for (Nat type = from->readNat(); type != endId; type = from->readNat()) {
+				checkTypeDescSize(sizeof(Member));
 				result->members->push(Member(null, type));
 			}
 
 			validateTuple(result);
 		} else if (flags & typeInfo::maybe) {
 			// Maybe-type.
-			result->members->push(Member(null, boolId));
-			result->members->push(Member(null, from->readNat()));
+			checkTypeDescSize(sizeof(Member)*2);
+			result->members->push(Member((Str *)null, boolId));
+			result->members->push(Member((Str *)null, from->readNat()));
 
 			validateMaybe(result);
 		} else if (flags & typeInfo::custom) {
@@ -480,7 +649,8 @@ namespace storm {
 		} else {
 			// Members.
 			for (Nat type = from->readNat(); type != endId; type = from->readNat()) {
-				Str *name = Str::read(from);
+				Str *name = Str::read(from, typeDescLimit - typeDescCurrent);
+				checkTypeDescSize(sizeof(Member) + name->peekLength()*sizeof(wchar));
 				result->members->push(Member(name, type));
 			}
 
@@ -494,7 +664,7 @@ namespace storm {
 	void ObjIStream::validateMembers(Desc *stream) {
 		SerializedStdType *our = as<SerializedStdType>(stream->info);
 		if (!our)
-			throw new (this) SerializationError(S("Trying to deserialize a standard type into a non-compatible type!"));
+			throw new (this) SerializationFormatError(S("Trying to deserialize a standard type into a non-compatible type!"));
 
 		// Note: We check the parent type when reading objects. Otherwise, we need quite a bit of
 		// bookkeeping to know when to validate parents of all types unless we want to check all
@@ -503,6 +673,10 @@ namespace storm {
 		// Note: We can not check the types of members here, as all member types are not necessarily
 		// known at this point. This is instead done when 'readXxx' is called.
 
+		// Note: If the stream contains a member that is not present in the type, it will be stored
+		// in temporary storage, but never used. This is fine, as we clear the temporary storage
+		// fairly quickly, and we expect this case to be fairly rare.
+
 		// Check the members we need to find, and match them to the members in the stream. During
 		// the process, figure out how to use the temporary storage to store some members there if
 		// necessary.
@@ -517,12 +691,17 @@ namespace storm {
 			// Look until we find it in the stream, saving intermediate members to temporary storage.
 			while (streamPos < memberCount) {
 				Member &m = stream->members->at(streamPos);
-				if (*m.name == *ourMember.name)
+				Str *mName = m.name();
+				if (mName && *mName == *ourMember.name)
 					break;
 
 				// Store it in temporary storage and remember its location (indexed from 1).
-				m.read = -1;
-				tempPos->put(m.name, Member(m, tempPos->count() + 1));
+				// We could try to figure out which members can be ignored already, but that
+				// is probably excessive as we expect changes in the data format to be fairly
+				// rare, and the cost is not too large here. One potential problem is if the
+				// object is entirely empty, and we only have -3 values everywhere.
+				m.read = -2;
+				tempPos->put(mName, Member(m, tempPos->count() + 1));
 				streamPos++;
 			}
 
@@ -533,16 +712,26 @@ namespace storm {
 			} else if (tempPos->has(ourMember.name)) {
 				// If we skipped it earlier, read it from temporary storage.
 				stream->members->push(tempPos->get(ourMember.name));
+			} else if (ourMember.init) {
+				// Use the initializer!
+				stream->members->push(Member(ourMember.init));
 			} else {
-				// Otherwise, an error.
-
-				// TODO: When we have support for default initialization, take that into consideration here!
-				Str *msg = TO_S(this, S("The member ") << ourMember.name << S(", required for type ")
-								<< runtime::typeName(our->type) << S(", is not present in the stream."));
-				throw new (this) SerializationError(msg);
+				// Otherwise, we are out of luck.
+				StrBuf *msg = new (this) StrBuf();
+				*msg << S("The member ") << ourMember.name << S(", required for type ")
+					 << runtime::typeName(our->type) << S(", is not present in the stream ")
+					 << S("and has no default value in the source code.");
+				throw new (this) SerializationFormatError(msg->toS());
 			}
 		}
 
+		// Catch extra members that are present at the end of the stream, but not in our type.
+		for (; streamPos < memberCount; streamPos++) {
+			// Note: We need to use the special "ignore" flag at the end. Otherwise, deserialization
+			// will not work properly.
+			stream->members->at(streamPos).read = -3;
+		}
+
 		// Remember the number of bytes required in temporary storage.
 		stream->storage(tempPos->count());
 
@@ -550,7 +739,7 @@ namespace storm {
 		// for (Nat i = 0; i < stream->members->count(); i++) {
 		// 	Member t = stream->members->at(i);
 
-		// 	PLN(L"  " << t.name << L", " << t.type << L", " << t.read);
+		// 	PLN(L"  " << t.name() << L", " << t.type << L", " << t.read);
 		// }
 		// PLN(L"  (" << stream->storage() << L" temporary entries required)");
 	}
@@ -558,7 +747,7 @@ namespace storm {
 	void ObjIStream::validateTuple(Desc *stream) {
 		SerializedTuples *our = as<SerializedTuples>(stream->info);
 		if (!our)
-			throw new (this) SerializationError(S("Trying to deserialize a type type into a non-compatible type!"));
+			throw new (this) SerializationFormatError(S("Trying to deserialize a type type into a non-compatible type!"));
 
 		// We don't try to do anything intelligent here. We just check so that the number of
 		// elements in each tuple match.
@@ -566,14 +755,14 @@ namespace storm {
 		if (our->count() != tupleCount) {
 			Str *msg = TO_S(this, S("Tuple size mismatch. Stream: ") << tupleCount
 							<< S(", here: ") << our->count() << S("."));
-			throw new (msg) SerializationError(msg);
+			throw new (msg) SerializationFormatError(msg);
 		}
 	}
 
 	void ObjIStream::validateMaybe(Desc *stream) {
 		SerializedMaybe *our = as<SerializedMaybe>(stream->info);
 		if (!our)
-			throw new (this) SerializationError(S("Trying to deserialize a type into a non-compatible type!"));
+			throw new (this) SerializationFormatError(S("Trying to deserialize a type into a non-compatible type!"));
 
 		// Nothing more to verify.
 	}
@@ -646,7 +835,7 @@ namespace storm {
 
 		const Handle &h = runtime::typeHandle(type);
 		if (!h.serializedTypeFn)
-			throw new (this) SerializationError(TO_S(this, S("The type ") << runtime::typeName(type) << S(" is not serializable.")));
+			throw new (this) SerializationFormatError(TO_S(this, S("The type ") << runtime::typeName(type) << S(" is not serializable.")));
 
 		SerializedType *info = (*h.serializedTypeFn)();
 		serializedTypes->put((TObject *)type, info);
@@ -661,11 +850,11 @@ namespace storm {
 			// for classes.
 			// PVAR(runtime::typeName(expected));
 			Str *msg = TO_S(this, S("Unexpected value type during serialization. Expected: ") << runtime::typeName(expected));
-			throw new (this) SerializationError(msg);
+			throw new (this) SerializationFormatError(msg);
 		}
 
 		if (type->info() & typeInfo::classType)
-			throw new (this) SerializationError(S("Expected a class type, but a value type was provided!"));
+			throw new (this) SerializationFormatError(S("Expected a class type, but a value type was provided!"));
 
 		writeInfo(type);
 		return true;
@@ -681,7 +870,7 @@ namespace storm {
 			while (expectedDesc && expectedDesc->type != expected)
 				expectedDesc = findSerialized(expectedDesc->super());
 			if (!expectedDesc)
-				throw new (this) SerializationError(S("The provided type description does not match the serialized object."));
+				throw new (this) SerializationFormatError(S("The provided type description does not match the serialized object."));
 
 			writeInfo(expectedDesc);
 
@@ -706,7 +895,7 @@ namespace storm {
 		}
 
 		if ((type->info() & typeInfo::classType) != typeInfo::classType)
-			throw new (this) SerializationError(S("Expected a value type, but a class type was provided."));
+			throw new (this) SerializationFormatError(S("Expected a value type, but a class type was provided."));
 
 		writeInfo(type);
 		return true;
@@ -730,7 +919,7 @@ namespace storm {
 		} else {
 			SerializedType::Cursor &at = depth->last();
 			if (!at.any())
-				throw new (this) SerializationError(S("Trying to serialize too many fields."));
+				throw new (this) SerializationFormatError(S("Trying to serialize too many fields."));
 
 			if (at.isParent())
 				r = null;
@@ -746,11 +935,11 @@ namespace storm {
 
 	void ObjOStream::end() {
 		if (depth->empty())
-			throw new (this) SerializationError(S("Mismatched calls to startX during serialization!"));
+			throw new (this) SerializationFormatError(S("Mismatched calls to startX during serialization!"));
 
 		SerializedType::Cursor end = depth->last();
 		if (!end.atEnd())
-			throw new (this) SerializationError(S("Missing fields during serialization!"));
+			throw new (this) SerializationFormatError(S("Missing fields during serialization!"));
 
 		depth->pop();
 
