@@ -360,6 +360,7 @@ namespace storm {
 		return r;
 	}
 
+	template <bool useEscape>
 	static bool unescape(const wchar *&src, wchar *&out, Char extra, Char extra2) {
 		switch (src[1]) {
 		case 'n':
@@ -393,6 +394,13 @@ namespace storm {
 			*out++ = wchar((a << 4) | b);
 			return true;
 		}
+		case '\\':
+			if (useEscape) {
+				*out++ = '\\';
+				src++;
+				return true;
+			}
+			// fall thru
 		default:
 			if (extra.leading() != 0) {
 				if (src[1] == extra.leading() && src[2] == extra.trailing()) {
@@ -401,7 +409,7 @@ namespace storm {
 					src += 2;
 					return true;
 				}
-			} else if (src[1] == extra.trailing()) {
+			} else if (extra.trailing() != 0 && src[1] == extra.trailing()) {
 				*out++ = extra.trailing();
 				src++;
 				return true;
@@ -414,7 +422,7 @@ namespace storm {
 					src += 2;
 					return true;
 				}
-			} else if (src[1] == extra2.trailing()) {
+			} else if (extra2.trailing() != 0 && src[1] == extra2.trailing()) {
 				*out++ = extra2.trailing();
 				src++;
 				return true;
@@ -437,10 +445,10 @@ namespace storm {
 		GcArray<wchar> *buf = runtime::allocArray<wchar>(engine(), &wcharArrayType, data->count);
 		wchar *to = buf->v;
 
-		for (const wchar *from = data->v; *from; from++) {
+		for (const wchar *from = data->v; from < data->v + data->count - 1; from++) {
 			wchar ch = *from;
 			if (ch == '\\') {
-				if (!storm::unescape(from, to, extra, extra2))
+				if (!storm::unescape<true>(from, to, extra, extra2))
 					*to++ = '\\';
 			} else {
 				*to++ = ch;
@@ -450,6 +458,28 @@ namespace storm {
 		return new (this) Str(buf->v);
 	}
 
+	Str *Str::unescapeKeepBackslash(Char extra) const {
+		GcArray<wchar> *buf = runtime::allocArray<wchar>(engine(), &wcharArrayType, data->count);
+		wchar *to = buf->v;
+
+		for (const wchar *from = data->v; from < data->v + data->count - 1; from++) {
+			wchar ch = *from;
+			if (ch == '\\') {
+				if (!storm::unescape<false>(from, to, extra, Char())) {
+					*to++ = '\\';
+					if (from[1] == '\\') {
+						*to++ = '\\';
+						from++;
+					}
+				}
+			} else {
+				*to++ = ch;
+			}
+		}
+
+		return new (this) Str(buf->v);
+	}
+
 	static bool escape(Char ch, StrBuf *to, Char extra, Char extra2) {
 		if (ch == Char('\n')) {
 			*to << L"\\n";
@@ -463,6 +493,9 @@ namespace storm {
 		} else if (ch == Char('\v')) {
 			*to << L"\\v";
 			return true;
+		} else if (ch == Char('\\')) {
+			*to << L"\\\\";
+			return true;
 		} else if (ch == Char('\0')) {
 			*to << L"\\0";
 			return true;
@@ -713,9 +746,9 @@ namespace storm {
 
 	Str::Str(IStream *from) {
 		Nat count = from->readNat();
-		Buffer b = from->read(count);
+		Buffer b = from->fill(count);
 		if (!b.full())
-			throw new (this) SerializationError(S("Not enough data."));
+			throw new (this) SerializationFormatError(S("Not enough data."));
 
 		size_t sz = convert((char *)b.dataPtr(), count, NULL, 0);
 		data = runtime::allocArray<wchar>(from->engine(), &wcharArrayType, sz);
@@ -728,12 +761,47 @@ namespace storm {
 		return new (from) Str(from);
 	}
 
+	Str::Str(IStream *from, Nat limitBytes) {
+		Nat count = from->readNat();
+		if (count >= limitBytes / sizeof(wchar))
+			throw new (this) SizeLimitReached(S("a string"), count * sizeof(wchar), limitBytes);
+		Buffer b = from->fill(count);
+		if (!b.full())
+			throw new (this) SerializationFormatError(S("Not enough data."));
+
+		size_t sz = convert((char *)b.dataPtr(), count, NULL, 0);
+		data = runtime::allocArray<wchar>(from->engine(), &wcharArrayType, sz);
+		convert((char *)b.dataPtr(), count, data->v, sz);
+
+		validate();
+	}
+
+	Str *Str::read(IStream *from, Nat limit) {
+		return new (from) Str(from, limit);
+	}
+
 	void Str::write(ObjOStream *to) const {
 		to->startPrimitive(strId);
 		write(to->to);
 		to->end();
 	}
 
+	Str::Str(ObjIStream *from) {
+		Nat count = from->from->readNat();
+		from->checkArrayAlloc(sizeof(wchar), count);
+		Buffer b = from->from->fill(count);
+		if (!b.full())
+			throw new (this) SerializationFormatError(S("Not enough data."));
+
+		size_t sz = convert((char *)b.dataPtr(), count, NULL, 0);
+		data = runtime::allocArray<wchar>(from->engine(), &wcharArrayType, sz);
+		convert((char *)b.dataPtr(), count, data->v, sz);
+
+		from->end();
+
+		validate();
+	}
+
 	Str *Str::read(ObjIStream *from) {
 		return (Str *)from->readPrimitiveObject(strId);
 	}
@@ -785,6 +853,39 @@ namespace storm {
 		return tmp;
 	}
 
+	Nat Str::Iter::operator -(const Iter &o) const {
+		const Str *owner = this->owner;
+		Nat thisPos = this->pos;
+		Nat otherPos = o.pos;
+
+		// If one of the iterators is the at-end operator, make it into a "proper" end for the
+		// string in the other iterator.
+		if (!owner)
+			owner = o.owner;
+		if (!owner)
+			return 0;
+
+		// Now, the other owner must either be null or the same as 'owner' now. Otherwise, we are
+		// comparing different strings.
+		if (o.owner != owner && o.owner != null)
+			return 0;
+
+		if (!this->owner)
+			thisPos = owner->data->count - 1;
+		if (!o.owner)
+			otherPos = owner->data->count - 1;
+
+		Nat diff = 0;
+		for (Nat at = otherPos; at < thisPos; diff++) {
+			if (utf16::leading(owner->data->v[at]))
+				at += 2;
+			else
+				at += 1;
+		}
+
+		return diff;
+	}
+
 	Bool Str::Iter::operator ==(const Iter &o) const {
 		if (atEnd() || o.atEnd())
 			return atEnd() == o.atEnd();
