@@ -8,6 +8,7 @@
 #include "GtkSignal.h"
 #include "GtkEmpty.h"
 #include "Env.h"
+#include "Exception.h"
 
 // We're using gtk_widget_override_font, as there is no better way of setting custom fonts on widgets...
 #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
@@ -21,10 +22,14 @@ namespace gui {
 		myHandle(invalid), myParent(null), myRoot(null),
 		myVisible(true), myEnabled(true), drawing(false), mouseInside(false),
 		gdkWindow(null), gTimer(null),
-		myPos(0, 0, 40, 40) /* large enought to not generate warnings in Gtk+ */ {
+		myPos(0, 0, 100, 100) /* large enough to not generate warnings in Gtk+ */ {
+
+		App *a = app(engine());
+		if (a->headless())
+			throw new (this) GuiError(S("Can not create windows when running in headless mode."));
 
 		myText = new (this) Str(L"");
-		myFont = app(engine())->defaultFont;
+		myFont = a->defaultFont;
 	}
 
 	Window::~Window() {
@@ -158,6 +163,10 @@ namespace gui {
 		// Nothing here, override when needed.
 	}
 
+	void Window::resized() {
+		resized(pos().size());
+	}
+
 	void Window::onResize(Size size) {
 		resized(size);
 	}
@@ -420,6 +429,10 @@ namespace gui {
 		return false;
 	}
 
+	bool Window::onNotify(NMHDR *data) {
+		return false;
+	}
+
 	Nat Window::currentDpi() {
 		if (myRoot)
 			return myRoot->currentDpi();
@@ -613,7 +626,8 @@ namespace gui {
 
 
 	bool Window::create(ContainerBase *parent, nat id) {
-		initWidget(parent, empty_new());
+		// Create an event box so that we receive events for this window.
+		initWidget(parent, gtk_event_box_new());
 		return true;
 	}
 
@@ -681,6 +695,7 @@ namespace gui {
 			return;
 
 		// Note: This might not be entirely correct in nested structures.
+		// Note: Seems to work OK inside a tab control, which seems to consist of multiple windows. Not entirely sure though.
 
 		for (GtkWidget *at = widget; !gtk_widget_get_has_window(at); at = gtk_widget_get_parent(at)) {
 			GtkAllocation delta;
@@ -691,52 +706,65 @@ namespace gui {
 		}
 	}
 
-	// Translate window coordinates to widget coordinates.
-	static bool translatePoint(GtkWidget *eventWidget, GtkWidget *to, gdouble x, gdouble y, Point &out) {
-		GtkAllocation alloc;
-		gtk_widget_get_allocation(to, &alloc);
+	// Translate window coordinates to widget coordinates. Note: Due to how Gtk propagates events,
+	// it might be the case that 'eventWindow' is a child window to whatever window 'to' might be inside.
+	static bool translatePoint(GdkWindow *eventWindow, GtkWidget *to, gdouble x, gdouble y, Point &out) {
+		// Find a widget that has a window by traversing from 'to'.
+		GtkWidget *toParent = to;
+		while (!gtk_widget_get_has_window(toParent)) {
+			toParent = gtk_widget_get_parent(toParent);
 
-		// Make relative to 'to's window.
-		x -= alloc.x;
-		y -= alloc.y;
-
-		// Check if there are any other windows on the way.
-		for (GtkWidget *at = gtk_widget_get_parent(to);
-			 at != null && at != eventWidget;
-			 at = gtk_widget_get_parent(at)) {
-
-			if (gtk_widget_get_has_window(at)) {
-				GtkAllocation diff;
-				gtk_widget_get_allocation(at, &diff);
-
-				// New window, update.
-				x -= diff.x;
-				x -= diff.y;
+			if (!toParent) {
+				WARNING(L"Failed to find a parent widget with a window!");
+				return false;
 			}
 		}
 
-		// If 'eventWidget' does not have a window, we need to update coordinates.
-		if (eventWidget && !gtk_widget_get_has_window(eventWidget)) {
-			GtkAllocation diff;
-			gtk_widget_get_allocation(eventWidget, &diff);
+		GdkWindow *toWindow = gtk_widget_get_window(toParent);
 
-			x += diff.x;
-			y += diff.y;
+		// At this point, we know that 'toParent' is a parent of 'eventWindow'. Try to find it by
+		// traversing from 'eventWindow', and updating coordinates as we go. GdkWindow coordinates
+		// are relative to the parent window, in contrast to widgets.
+		while (eventWindow != toWindow) {
+			// Make coordinates relative the window's parent.
+			gint winX, winY;
+			gdk_window_get_position(eventWindow, &winX, &winY);
+			x += winX;
+			y += winY;
+
+			// Traverse.
+			eventWindow = gdk_window_get_effective_parent(eventWindow);
+
+			if (!eventWindow) {
+				// This happens as it seems like events for a window are passed to the dialog window
+				// first. As such, we should not warn in this case.
+				return false;
+			}
 		}
 
+		// Now, we have coordinates that are relative to the window owned by "toParent". Now we can
+		// just ask Gtk to convert coordinates for us!
+		gint xDelta, yDelta;
+		gtk_widget_translate_coordinates(toParent, to, 0, 0, &xDelta, &yDelta);
+		x += xDelta;
+		y += yDelta;
+
+		// Finally, we can ask Gtk about the allocation of 'to' and make sure that the coordinate is
+		// inside the rectangle.
+		GtkAllocation toAlloc;
+		gtk_widget_get_allocation(to, &toAlloc);
+
 		out.x = x;
 		out.y = y;
 
-		// PLN(L"Result: " << out.x << ", " << out.y);
-
 		return out.x >= 0.0f && out.y >= 0.0f
-			&& out.x < alloc.width && out.y < alloc.height;
+			&& out.x < toAlloc.width && out.y < toAlloc.height;
 	}
 
 	gboolean Window::onButton(GdkEvent *event) {
 		Point pt;
 		GdkEventButton &b = event->button;
-		if (!translatePoint(handle().widget(), drawWidget(), b.x, b.y, pt))
+		if (!translatePoint(b.window, drawWidget(), b.x, b.y, pt))
 			return FALSE;
 
 		mouse::MouseButton button = mouse::MouseButton(b.button - 1);
@@ -761,7 +789,7 @@ namespace gui {
 	gboolean Window::onMotion(GdkEvent *event) {
 		Point pt;
 		GdkEventMotion &m = event->motion;
-		if (!translatePoint(handle().widget(), drawWidget(), m.x, m.y, pt))
+		if (!translatePoint(m.window, drawWidget(), m.x, m.y, pt))
 			return FALSE;
 
 		return onMouseMove(pt) ? TRUE : FALSE;
@@ -784,28 +812,37 @@ namespace gui {
 	gboolean Window::onScroll(GdkEvent *event) {
 		Point pt;
 		GdkEventScroll &s = event->scroll;
-		if (!translatePoint(handle().widget(), drawWidget(), s.x, s.y, pt))
+		if (!translatePoint(s.window, drawWidget(), s.x, s.y, pt)) {
 			return FALSE;
-
-		gdouble dx = 0, dy = 0;
-		if (!gdk_event_get_scroll_deltas(event, &dx, &dy)) {
-			dx = 120; dy = 120;
 		}
 
+		const Int step = 120;
+
 		bool ok = false;
-		switch (s.direction) {
-		case GDK_SCROLL_UP:
-			ok = onMouseVScroll(pt, Int(dx));
-			break;
-		case GDK_SCROLL_DOWN:
-			ok = onMouseVScroll(pt, -Int(dx));
-			break;
-		case GDK_SCROLL_LEFT:
-			ok = onMouseHScroll(pt, -Int(dy));
-			break;
-		case GDK_SCROLL_RIGHT:
-			ok = onMouseHScroll(pt, Int(dy));
-			break;
+		GdkScrollDirection direction;
+		gdouble dx = 0, dy = 0;
+		if (gdk_event_get_scroll_direction(event, &direction)) {
+			switch (s.direction) {
+			case GDK_SCROLL_UP:
+				ok = onMouseVScroll(pt, -step);
+				break;
+			case GDK_SCROLL_DOWN:
+				ok = onMouseVScroll(pt, step);
+				break;
+			case GDK_SCROLL_LEFT:
+				ok = onMouseHScroll(pt, -step);
+				break;
+			case GDK_SCROLL_RIGHT:
+				ok = onMouseHScroll(pt, step);
+				break;
+			}
+		} else if (gdk_event_get_scroll_deltas(event, &dx, &dy)) {
+			Int x(dx * step);
+			if (x != 0)
+				ok |= onMouseHScroll(pt, -x);
+			Int y(dy * step);
+			if (y != 0)
+				ok |= onMouseVScroll(pt, -y);
 		}
 		return ok ? TRUE : FALSE;
 	}
@@ -830,18 +867,27 @@ namespace gui {
 		Size s(drawAlloc.width, drawAlloc.height);
 		myPos.size(s);
 
-		if (myPainter) {
-			int scale = gtk_widget_get_scale_factor(draw);
-			myPainter->uiResize(s * scale, scale);
-		}
-		onResize(s);
+		// GTK sends us this signal quite often, so filter out dummy draws.
+		if (lastWidth != drawAlloc.width || lastHeight != drawAlloc.height) {
+			lastWidth = drawAlloc.width;
+			lastHeight = drawAlloc.height;
 
-		// Queue a repaint for this window. Otherwise, resizing windows that partially contain some
-		// continuous rendering will not always be updated properly. We're probably in a race with
-		// Gtk when we just update a part of the window.
-		GdkWindow *window = gtk_widget_get_window(handle().widget());
-		if (window) {
-			gdk_window_invalidate_rect(window, NULL, false);
+			if (myPainter) {
+				int scale = gtk_widget_get_scale_factor(draw);
+				myPainter->uiResize(s * scale, scale);
+			}
+			onResize(s);
+
+			// Queue a repaint for this window.
+			gtk_widget_queue_draw(handle().widget());
+
+			// // Queue a repaint for this window. Otherwise, resizing windows that partially contain some
+			// // continuous rendering will not always be updated properly. We're probably in a race with
+			// // Gtk when we just update a part of the window.
+			// GdkWindow *window = gtk_widget_get_window(handle().widget());
+			// if (window) {
+			// 	gdk_window_invalidate_rect(window, NULL, false);
+			// }
 		}
 	}
 
@@ -1185,19 +1231,21 @@ namespace gui {
 
 	void Window::update() {
 		if (created()) {
-			GdkWindow *window = gtk_widget_get_window(handle().widget());
-			if (window) {
-				gdk_window_invalidate_rect(window, NULL, true);
-				gdk_window_process_updates(window, true);
-			}
+			gtk_widget_queue_draw(handle().widget());
+			// GdkWindow *window = gtk_widget_get_window(handle().widget());
+			// if (window) {
+			// 	gdk_window_invalidate_rect(window, NULL, true);
+			// 	gdk_window_process_updates(window, true);
+			// }
 		}
 	}
 
 	void Window::repaint() {
 		if (created()) {
-			GdkWindow *window = gtk_widget_get_window(handle().widget());
-			if (window)
-				gdk_window_invalidate_rect(window, NULL, true);
+			gtk_widget_queue_draw(handle().widget());
+			// GdkWindow *window = gtk_widget_get_window(handle().widget());
+			// if (window)
+			// 	gdk_window_invalidate_rect(window, NULL, true);
 		}
 	}
 
