@@ -33,8 +33,8 @@
 (defvar-local storm-buffer-edit-id 0 "The ID of the next edit operation for the current buffer.")
 (defvar-local storm-buffer-edits nil
   "List of the last few edits to this buffer. This is so we can handle 'late' coloring messages from Storm.
-   The entries of this list has the form (offset skip marker) where offset is the character offset where the edit started,
-   skip is the number of characters erased (if any) and marker is a marker placed at offset + skip.")
+   The entries of this list has the form (offset skip end) where offset is the character offset where the edit started,
+   skip is the number of characters erased (if any) and end is the offset where the insertion ended.")
 (defvar-local storm-buffer-no-changes nil "Inhibit change notifications for the current buffer.")
 (defvar-local storm-buffer-last-point 0 "Remembers where Storm thinks the point is located in this buffer.")
 
@@ -48,11 +48,6 @@
 
 ;; Mark errors in compilation-mode properly.
 (require 'compile)
-(require 'cl)
-
-;; Compilation mode assumes that error messages are in screen columns. This is typically not the
-;; case (for C++ or Storm).
-(setq-default compilation-error-screen-columns nil)
 
 (defun storm-find-error-buffer (full-file)
   (let ((buffer (find-buffer-visiting full-file)))
@@ -64,23 +59,23 @@
 
 ;; compile.el was updated in emacs 27. In the new version, the old way of adding custom hooks no
 ;; longer work properly. Instead, it provides a documented way of adding hooks.
-(pushnew (if (> emacs-major-version 26)
-	     ;; For new emacs
-	     `(storm ,storm-error-regexp
-		     2 ;; File
-		     (storm-compute-line-begin . storm-compute-line-end) ;; Lines
-		     (storm-compute-col-begin . storm-compute-col-end) ;; Columns
-		     2 ;; Error type
-		     )
-	   ;; For old emacs
-	   `(storm ,storm-error-regexp
-		  2 ;; File
-		  storm-old-compute-line ;; Function to call (line)
-		  3 ;; Column (likely not used due to hook)
-		  nil ;; Type (likely not used due to hook)
-		  )
-	   )
-	 compilation-error-regexp-alist-alist)
+(cl-pushnew (if (> emacs-major-version 26)
+		;; For new emacs
+		`(storm ,storm-error-regexp
+			2 ;; File
+			(storm-compute-line-begin . storm-compute-line-end) ;; Lines
+			(storm-compute-col-begin . storm-compute-col-end) ;; Columns
+			2 ;; Error type
+			)
+	      ;; For old emacs
+	      `(storm ,storm-error-regexp
+		      2 ;; File
+		      storm-old-compute-line ;; Function to call (line)
+		      3 ;; Column (likely not used due to hook)
+		      nil ;; Type (likely not used due to hook)
+		      )
+	      )
+	    compilation-error-regexp-alist-alist)
 (add-to-list 'compilation-error-regexp-alist 'storm)
 
 ;; Old-style lookup function. Only works for emacs <= 25.
@@ -142,7 +137,11 @@
       (save-excursion
 	(setq offset (1+ (string-to-number offset)))
 	(goto-char offset)
-	(+ (- offset (line-beginning-position)) compilation-first-column)))))
+	(if compilation-error-screen-columns
+	    (let ((tab-width 8))
+	      ;; If we are in "visual columns"-mode, use tab-width = 8, since that is what compilation-mode uses internally.
+	      (+ (current-column) compilation-first-column))
+	  (+ (- offset (line-beginning-position)) compilation-first-column))))))
 
 (defun storm-compute-col-begin ()
   (storm-compute-col (match-string-no-properties 2) (match-string-no-properties 3)))
@@ -247,9 +246,9 @@
     (when (listp response)
       (setq storm-buffer-last-point (point))
       (when (>= (length response) 3)
-	(let ((file-id (first  response))
-	      (kind    (second response))
-	      (value   (third  response)))
+	(let ((file-id (nth 0 response))
+	      (kind    (nth 1 response))
+	      (value   (nth 2 response)))
 	  (cond ((not (eq storm-buffer-id file-id))
 		 'noindent)
 		((eq kind 'level)
@@ -280,9 +279,20 @@
 ;; Commands.
 
 (defun storm-insert-indent (arg)
+  "Insert character and update indentation after doing so. Much like electric-indent in Emacs."
   (interactive "*P")
-  (self-insert-command (prefix-numeric-value arg))
-  (storm-indent-line))
+  ;; Note: If paren highlighting is active, the self-insert-command will block for some
+  ;; time. Because of this, we will temporarily unset the blink function and then call it manually
+  ;; later on.
+  (let ((old-blink-paren blink-paren-function)
+	(blink-paren-function nil))
+    (self-insert-command (prefix-numeric-value arg))
+    (storm-indent-line)
+    ;; Blink paren if applicable.
+    (and (eq (char-syntax (c-last-command-char)) ?\)) ;; is it a close paren character?
+	 old-blink-paren ;; something to call?
+	 (save-excursion
+	  (funcall old-blink-paren)))))
 
 (defun storm-debug-tree ()
   "Output debug information containing the syntax tree for the current buffer."
@@ -374,12 +384,17 @@
     (setq storm-buffer-edits nil)
     (setq storm-buffer-last-point (point))
 
-    ;; Tell Storm what is happening.
-    (storm-send (list 'open
-		      storm-buffer-id
-		      (buffer-file-name)
-		      (buffer-substring-no-properties (point-min) (point-max))
-		      (point)))))
+    ;; Tell Storm what is happening. If the Storm process is not running, just start it now.  After
+    ;; the process is started, this function will be called again to register the buffer.  As such,
+    ;; we don't queue a message if it is not started as that would result in two open messages for
+    ;; the same buffer.
+    (if (storm-running-p)
+	(storm-send (list 'open
+			  storm-buffer-id
+			  (buffer-file-name)
+			  (buffer-substring-no-properties (point-min) (point-max))
+			  (point)))
+      (storm-start))))
 
 (defun storm-buffer-killed ()
   "Called when a buffer is going to be killed."
@@ -397,15 +412,14 @@
   NOTE: This is called when we set text color of the buffer as well... Inhibit that!"
   (when (and storm-buffer-id (not storm-buffer-no-changes))
     ;; Remember the edit.
-    (let ((marker (make-marker)))
-      (setq storm-buffer-edit-id (1+ storm-buffer-edit-id))
-      (setq storm-buffer-edits
-	    (storm-limit-edit-length
-	     storm-mode-max-edits
-	     (cons (list edit-begin
-			 old-length
-			 (set-marker marker edit-end))
-		   storm-buffer-edits))))
+    (setq storm-buffer-edit-id (1+ storm-buffer-edit-id))
+    (setq storm-buffer-edits
+	  (storm-limit-edit-length
+	   storm-mode-max-edits
+	   (cons (list edit-begin
+		       old-length
+		       edit-end)
+		 storm-buffer-edits)))
 
     ;; Tell Storm.
     (setq storm-buffer-last-point edit-begin)
@@ -427,19 +441,13 @@
 (defun storm-limit-edit-length (max-length list)
   "Limit the length of the list 'list' containing edits."
   (let ((at list)
-	(pos 1)
-	(free nil))
+	(pos 1))
     (while (consp at)
       (when (= pos max-length)
-	(setq free (cdr at))
 	(setcdr at nil))
       (setq at (cdr at))
       (setq pos (1+ pos)))
 
-    ;; Remove any markers so that they do not slow things down...
-    (while (consp free)
-      (set-marker (third (car free)) nil)
-      (setq free (cdr free)))
     list))
 
 
@@ -462,15 +470,15 @@
 (defmacro storm-check-edits (pos min-pos max-pos edits)
   "Update 'pos' according to the history in 'edits'. Consumes entries from 'edits'."
   `(while (and (consp ,edits)
-	       (<= (first (first ,edits)) ,pos))
-     (let* ((entry (first ,edits))
-	    (offset (first entry))
-	    (skip (second entry))
-	    (marker (third entry)))
+	       (<= (car (car ,edits)) ,pos))
+     (let* ((entry (car ,edits))
+	    (offset (nth 0 entry))
+	    (skip   (nth 1 entry))
+	    (end    (nth 2 entry)))
        (when (> skip 0)
-	 (setq ,min-pos (min (first entry) ,max-pos)))
-       (setq ,pos (+ (marker-position marker) (- ,pos offset skip)))
-       (setq ,edits (rest ,edits)))))
+	 (setq ,min-pos (min (car entry) ,max-pos)))
+       (setq ,pos (+ end (- ,pos offset skip)))
+       (setq ,edits (cdr ,edits)))))
 
 (defun storm-on-color (params)
   "Handle the color-message."
@@ -482,8 +490,7 @@
 	     (buffer    (gethash buffer-id storm-mode-buffers nil)))
 	(when buffer
 	  (with-current-buffer buffer
-	    ;; Make sure we do not act on our own color notifications!
-	    (let ((storm-buffer-no-changes t)
+	    (let ((storm-buffer-no-changes t) ;; To not send changes for color updates!
 		  (edits   (storm-active-edits edit-id))
 		  (lowest  (point-min))
 		  (highest (point-max))
@@ -491,13 +498,13 @@
 
 	      (storm-check-edits start-pos lowest highest edits)
 	      (while (consp at)
-		(setq end-pos (+ start-pos (first at)))
+		(setq end-pos (+ start-pos (car at)))
 		(storm-check-edits end-pos lowest highest edits)
 
 		(storm-set-color
 		 (max lowest start-pos)
 		 (min highest end-pos)
-		 (storm-find-color (second at)))
+		 (storm-find-color (cadr at)))
 
 		(setq start-pos end-pos)
 		(setq at (nthcdr 2 at)))))))
@@ -510,8 +517,8 @@
       (if (eq old 'unknown)
 	  (let ((r (storm-query (list 'supported ext))))
 	    (when (listp r)
-	      (puthash ext (second r) storm-mode-types)
-	      (second r)))
+	      (puthash ext (cadr r) storm-mode-types)
+	      (cadr r)))
 	old))))
 
 
@@ -605,7 +612,7 @@
   "Send a message to the Storm process and await a result for a maximum of 1 second. The 
    response is expected to start with the same symbol as the sent message."
 
-  (setq storm-process-query (first message))
+  (setq storm-process-query (car message))
   (storm-send message)
 
   (unless timeout
@@ -657,9 +664,11 @@
   (add-to-list 'compilation-finish-functions 'storm-compilation-finished)
   ;; Make sure not to run multiple compilations.
   (unless storm-running-compile
-    (let ((default-directory (storm-parent-directory (file-name-directory storm-mode-compiler))))
+    (let* ((default-directory (storm-parent-directory (file-name-directory storm-mode-compiler)))
+	   (mm-command (if (null mymake-command) "mymake" mymake-command))
+	   (command (concat mm-command " Main -ne")))
       (setq storm-running-compile
-	    (compilation-start "mm Main -ne")))))
+	    (compilation-start command)))))
 
 (defun storm-compilation-finished (buffer exit-status)
   "Called whenever a compilation buffer has finished."
@@ -677,14 +686,14 @@
   (storm-include-params-i storm-mode-include))
 
 (defun storm-include-params-i (list)
-  (if (endp list)
+  (if (null list)
       'nil
-    (let* ((first (first list))
+    (let* ((first (car list))
 	   (path  (car first))
 	   (pkg   (cdr first)))
       (append
        (list "-I" pkg (expand-file-name path))
-       (storm-include-params-i (rest list))))))
+       (storm-include-params-i (cdr list))))))
 
 (defun storm-start-compiler ()
   "Start the Storm compiler."
@@ -769,7 +778,7 @@
 (defun storm-decode-messages (string)
   "Decode all messages in 'string'. Returns (t remaining) if we managed to parse something,
    or (nil remaining) if we failed parsing things."
-  (let ((null-char (position #x00 string))
+  (let ((null-char (cl-position #x00 string))
 	(failed 'nil))
     (while (and (not failed)
 		(numberp null-char))
@@ -785,7 +794,7 @@
 		(message (cdr result)))
 	    (setq string (substring string consumed))
 	    (storm-handle-message message)
-	    (setq null-char (position #x00 string))))))
+	    (setq null-char (cl-position #x00 string))))))
 
     ;; Figure out the result.
     (if failed
@@ -802,16 +811,16 @@
 (defun storm-handle-message-i (msg)
   "Handle a message"
   (if (consp msg)
-      (let* ((header (first msg))
+      (let* ((header (car msg))
 	     (found (gethash header storm-messages)))
 	(when (eq header storm-process-wait)
 	  (setq storm-process-wait nil))
 	(if (eq header storm-process-query)
 	    (progn
-	      (setq storm-process-query (rest msg))
+	      (setq storm-process-query (cdr msg))
 	      t)
 	  (if found
-	      (funcall found (rest msg))
+	      (funcall found (cdr msg))
 	    (format "The header %S is not supported." header))))
     "Invalid message format. Expected a list."))
 
@@ -1152,9 +1161,9 @@
 		  (list 'documentation name default-directory)
 		name))
 	 (doc (storm-query msg 3))) ;; Allow 3 s timeout for compilation.
-    (if (or (null doc) (null (first doc)))
+    (if (or (null doc) (null (car doc)))
 	(message "\"%s\" does not exist." name)
-      (storm-show-doc msg (first doc) no-history))))
+      (storm-show-doc msg (car doc) no-history))))
 
 
 (defun storm-show-doc (msg doc no-history)
@@ -1175,7 +1184,7 @@
   "Limit the length of 'seq' to 'max' elements."
   (if (<= max 0)
       nil
-    (if (endp seq)
+    (if (null seq)
 	nil
       (cons (car seq)
 	    (limit (cdr seq) (1- max))))))
@@ -1194,15 +1203,15 @@
 	(storm-insert-link (cdr vis) (car vis) nil)
 	(insert " "))
       (storm-insert-face name 'bold)
-      (unless (endp params)
+      (unless (null params)
 	(insert "(")
-	(storm-insert-param (first params))
-	(mapc (lambda (x) (insert ", ") (storm-insert-param x)) (rest params))
+	(storm-insert-param (car params))
+	(mapc (lambda (x) (insert ", ") (storm-insert-param x)) (cdr params))
 	(insert ")"))
-      (unless (endp notes)
+      (unless (null notes)
 	(insert " ")
-	(storm-insert-note (first notes))
-	(mapc (lambda (x) (insert ", ") (storm-insert-note x)) (rest notes)))
+	(storm-insert-note (car notes))
+	(mapc (lambda (x) (insert ", ") (storm-insert-note x)) (cdr notes)))
       (insert ":\n\n")
       (when pos
 	(insert "At: ")
@@ -1258,16 +1267,16 @@
 (defun storm-doc-fwd ()
   (interactive)
   (when storm-doc-next
-    (setq storm-doc-prev (cons (first storm-doc-next) (limit storm-doc-prev storm-doc-history)))
-    (setq storm-doc-next (rest storm-doc-next))
-    (storm-doc (first storm-doc-prev) t)))
+    (setq storm-doc-prev (cons (car storm-doc-next) (limit storm-doc-prev storm-doc-history)))
+    (setq storm-doc-next (cdr storm-doc-next))
+    (storm-doc (car storm-doc-prev) t)))
 
 (defun storm-doc-back ()
   (interactive)
   (when (and storm-doc-prev (> (length storm-doc-prev) 1))
-    (setq storm-doc-next (cons (first storm-doc-prev) (limit storm-doc-next storm-doc-history)))
-    (setq storm-doc-prev (rest storm-doc-prev))
-    (storm-doc (first storm-doc-prev) t)))
+    (setq storm-doc-next (cons (car storm-doc-prev) (limit storm-doc-next storm-doc-history)))
+    (setq storm-doc-prev (cdr storm-doc-prev))
+    (storm-doc (car storm-doc-prev) t)))
 
 
 (defun storm-insert-param (param)
@@ -1302,7 +1311,7 @@
 	  (storm-insert-link type-name type-ref t))
 	(when ref
 	  (insert "&")))
-    (insert (first param))))
+    (insert (car param))))
 
 (defvar storm-doc-history 20 "Number of history entries for documentation in Storm.")
 (defvar storm-doc-prev nil "History for the documentation from Storm.")
